@@ -2,7 +2,7 @@
 /*
  *  md5.h
  *
- *  Copyright (c) 2004 - 2008, clown. All rights reserved.
+ *  Copyright (c) 2004 - 2010, clown. All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions
@@ -36,7 +36,7 @@
  *  Message-Digest Algorithm. (See the following URL:
  *  http://www.faqs.org/rfcs/rfc1321.html)
  *
- *  Last-modified: Sat 14 Mar 2009 21:19:00 JST
+ *  Last-modified: Thu 28 Jan 2010 05:39:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_MD5_H
@@ -48,6 +48,7 @@
 #include <string>
 #include <sstream>
 #include "memory.h"
+#include "rotate.h"
 
 // F, G, H and I are basic MD5 functions.
 #define F(x, y, z) (((x) & (y)) | ((~x) & (z)))
@@ -55,28 +56,24 @@
 #define H(x, y, z) ((x) ^ (y) ^ (z))
 #define I(x, y, z) ((y) ^ ((x) | (~z)))
 
-#ifndef ROTATE_LEFT
-#define ROTATE_LEFT(x, n) (((x) << (n)) | ((x) >> (32 - n)))
-#endif
-
 #define ROUND1(a, b, c, d, x, s, ac) { \
  (a) += F((b), (c), (d)) + (x) + (uint32_t)(ac); \
- (a) = ROTATE_LEFT((a), (s)); \
+ (a) = clx::rotate_copy((a), (s)); \
  (a) += (b); \
   }
 #define ROUND2(a, b, c, d, x, s, ac) { \
  (a) += G((b), (c), (d)) + (x) + (uint32_t)(ac); \
- (a) = ROTATE_LEFT ((a), (s)); \
+ (a) = clx::rotate_copy((a), (s)); \
  (a) += (b); \
   }
 #define ROUND3(a, b, c, d, x, s, ac) { \
  (a) += H((b), (c), (d)) + (x) + (uint32_t)(ac); \
- (a) = ROTATE_LEFT ((a), (s)); \
+ (a) = clx::rotate_copy((a), (s)); \
  (a) += (b); \
   }
 #define ROUND4(a, b, c, d, x, s, ac) { \
  (a) += I((b), (c), (d)) + (x) + (uint32_t)(ac); \
- (a) = ROTATE_LEFT ((a), (s)); \
+ (a) = clx::rotate_copy((a), (s)); \
  (a) += (b); \
   }
 
@@ -127,7 +124,7 @@ namespace clx {
 			w_[3] = 0x10325476;
 		}
 		
-		void update(const char_type* src, size_t n) {
+		void update(const char_type* src, std::size_t n) {
 			if (n == 0) return;
 			uchar_t* block = (uchar_t*)src;
 			
@@ -152,7 +149,7 @@ namespace clx {
 				block += 64;
 			}
 			
-			if (n > 0) clx::memcpy(buffer_ + left, block, n);
+			if (n > 0) clx::memcpy(buffer_ + left, block, static_cast<int>(n));
 		}
 		
 		void finish() {
@@ -177,7 +174,7 @@ namespace clx {
 			std::memset(buffer_, 0, sizeof(buffer_));
 		}
 		
-		int size() const { return sizeof(code_) - 1; }
+		std::size_t size() const { return sizeof(code_) - 1; }
 		const uchar_t* code() const { return code_; }
 		
 		string_type to_string() const {
