@@ -68,7 +68,7 @@ namespace clx {
 		explicit basic_scanner(const string_type& src, const string_type& fmt) :
 			f_(separator(fmt)) { this->scan(src); }
 		
-		~basic_scanner() {}
+		virtual ~basic_scanner() throw() {}
 		
 		template <class ValueT>
 		basic_scanner& operator%(ValueT& dest) {
