@@ -2,7 +2,7 @@
 /*
  *  smtp.h
  *
- *  Copyright (c) 2004 - 2008, clown. All rights reserved.
+ *  Copyright (c) 2004 - 2010, clown. All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions
@@ -29,7 +29,7 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  Last-modified: Thu 13 Nov 2008 23:41:00 JST
+ *  Last-modified: Mon 15 Mar 2010 23:45:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_SMTP_H
@@ -38,6 +38,7 @@
 #include <iostream>
 #include <string>
 #include <deque>
+#include <vector>
 #include <list>
 #include <stdexcept>
 #include "tcp.h"
@@ -115,18 +116,26 @@ namespace clx {
 		typedef std::pair<int, string_type> response_type;
 		typedef std::deque<response_type> response_array;
 		
-		basic_smtp() : socket_type(), host_(), port_(0), auth_(0), res_() {}
+		basic_smtp() :
+			socket_type(), host_(), port_(0), auth_(0), res_(),
+			sbuf_(sockbuf_type::nbuf) {}
 		
 		basic_smtp(const basic_smtp& cp) :
-			socket_type(cp), host_(cp.host_), port_(cp.port_), auth_(0), res_(cp.res_) {}
+			socket_type(cp), host_(cp.host_), port_(cp.port_),
+			auth_(0), res_(cp.res_),
+			sbuf_(sockbuf_type::nbuf) {}
 		
 		explicit basic_smtp(const string_type& host, int port = 25) :
-			socket_type(), host_(host), port_(port), auth_(0), res_() {
+			socket_type(), host_(host), port_(port),
+			auth_(0), res_(),
+			sbuf_(sockbuf_type::nbuf) {
 			this->start();
 		}
 		
 		explicit basic_smtp(const char_type* host, int port = 25) :
-			socket_type(), host_(host), port_(port), auth_(0), res_() {
+			socket_type(), host_(host), port_(port),
+			auth_(0), res_(),
+			sbuf_(sockbuf_type::nbuf) {
 			this->start();
 		}
 		
@@ -134,9 +143,9 @@ namespace clx {
 		
 		basic_smtp& start() {
 			if (!this->is_open()) this->connect(host_, port_);
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
-			this->xresponse(ss);
+			sbuf_.socket(*this);
+			std::basic_iostream<char_type> ss(&sbuf_);
+			this->response(ss);
 			return *this;
 		}
 		
@@ -153,20 +162,13 @@ namespace clx {
 		
 		void finish() {
 			if (this->is_open()) {
-				basic_sockbuf<socket_type, char_type> sbuf(*this);
-				std::basic_iostream<char_type> ss(&sbuf);
-				ss << "QUIT\r\n";
-				this->xresponse(ss);
+				this->command("QUIT");
 				this->close();
 			}
 		}
 		
 		basic_smtp& login(const string_type& id) {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
-			ss << ("EHLO " + id + " " + host_ + "\r\n");
-			this->xresponse(ss);
-			
+			this->command("EHLO " + id + " " + host_);
 			for (size_t i = 0; i < res_.size(); ++i) {
 				if (res_[i].first == 250 && res_[i].second.find("AUTH") != string_type::npos) {
 					if (res_[i].second.find("PLAIN") != string_type::npos) auth_ |= smtp_auth::plain;
@@ -174,7 +176,6 @@ namespace clx {
 					if (res_[i].second.find("CRAM-MD5") != string_type::npos) auth_ |= smtp_auth::cram_md5;
 				}
 			}
-			
 			return *this;
 		}
 		
@@ -186,9 +187,9 @@ namespace clx {
 		basic_smtp& login(const string_type& id, const string_type& pass, int type = smtp_auth::best) {
 			this->login(id);
 			
-			if ((type & auth_ & smtp_auth::cram_md5) != 0) return xcram_md5(id, pass);
-			else if ((type & auth_ & smtp_auth::plain) != 0) return xplain(id, pass);
-			else if ((type & auth_ & smtp_auth::login) != 0) return xlogin(id, pass);
+			if ((type & auth_ & smtp_auth::cram_md5) != 0) return this->auth_cram_md5(id, pass);
+			else if ((type & auth_ & smtp_auth::plain) != 0) return this->auth_plain(id, pass);
+			else if ((type & auth_ & smtp_auth::login) != 0) return this->auth_login(id, pass);
 			else if ((type & smtp_auth::none) != 0) return *this;
 			
 			throw smtp_error(502, "SMTP-AUTH not supported");
@@ -203,26 +204,18 @@ namespace clx {
 		
 		template <class MailingList, class Functor>
 		basic_smtp& mail(const string_type& from, const MailingList& to, Functor f) {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
+			std::basic_iostream<char_type> ss(&sbuf_);
 			string_type buf;
 			
-			buf = "MAIL FROM:<" + from + ">\r\n";
-			ss << buf;
-			this->xresponse(ss);
-			
+			this->command(ss, "MAIL FROM:<" + from + ">");
 			for (typename MailingList::const_iterator pos = to.begin(); pos != to.end(); ++pos) {
-				buf = "RCPT TO:<" + *pos + ">\r\n";
-				ss << buf;
-				this->xresponse(ss);
+				this->command(ss, "RCPT TO:<" + *pos + ">");
 			}
 			
-			ss << "DATA\r\n";
-			this->xresponse(ss);
-			
+			this->command(ss, "DATA");
 			f(ss);
-			this->xresponse(ss);
 			
+			this->response(ss);
 			return *this;
 		}
 		
@@ -248,82 +241,92 @@ namespace clx {
 		const response_type& response(size_t index) { return res_.at(index); }
 		
 	private:
+		typedef clx::basic_sockbuf<socket_type, char_type> sockbuf_type;
+		
 		string_type host_;
 		int port_;
 		int auth_;
 		response_array res_;
+		sockbuf_type sbuf_;
 		
 		template <class InputStream>
-		void xresponse(InputStream& sin) {
+		void response(InputStream& sin) {
+			std::basic_stringstream<char_type> dest;
 			string_type buf;
 			while (std::getline(sin, buf)) {
-				chomp(buf);
-				response_type tmp;
+				clx::chomp(buf);
+				int code;
 				
 				try {
-					tmp.first = lexical_cast<int>(buf.substr(0, 3));
+					code = clx::lexical_cast<int>(buf.substr(0, 3));
+					if (!dest.str().empty()) dest << std::endl;
+					dest << buf.substr(4);
 				}
 				catch (clx::bad_lexical_cast&) {
-					string_type msg = "invalid response: " + buf;
-					throw std::runtime_error(msg);
+					throw std::runtime_error("invalid response: " + buf);
 				}
 				
-				tmp.second = buf.substr(4);
-				res_.push_front(tmp);
-				if (tmp.first >= 400) throw smtp_error(tmp.first, tmp.second);
-				if (buf[3] == ' ') break;
+				if (buf[3] == ' ') {
+					if (code >= 400) throw clx::smtp_error(code, dest.str());
+					response_type tmp(code, dest.str());
+					res_.push_front(tmp);
+					break;
+				}
 			}
 		}
 		
-		basic_smtp& xplain(const string_type& id, const string_type& pass) {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
-			
-			int len = id.size() * 2 + pass.size() + 2;
-			char* p = new char[len + 1];
-			std::memset(p, 0, len + 1);
-			std::memcpy(p, id.c_str(), id.size());
-			std::memcpy(&p[id.size() + 1], id.c_str(), id.size());
-			std::memcpy(&p[(id.size() + 1) * 2], pass.c_str(), pass.size());
-			
-			ss << ("AUTH PLAIN " + base64::encode(p, len) + "\r\n");
-			delete [] p;
-			
-			this->xresponse(ss);
+		template <class IOStream>
+		basic_smtp& command(IOStream& ss, const string_type& cmd) {
+			ss << (cmd + "\r\n");
+			this->response(ss);
 			return *this;
 		}
 		
-		basic_smtp& xlogin(const string_type& id, const string_type& pass) {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
+		basic_smtp& command(const string_type& cmd) {
+			std::basic_iostream<char_type> ss(&sbuf_);
+			return this->command(ss, cmd);
+		}
+		
+		basic_smtp& auth_plain(const string_type& id, const string_type& pass) {
+			std::basic_iostream<char_type> ss(&sbuf_);
 			
-			ss << "AUTH LOGIN\r\n";
-			this->xresponse(ss);
-			ss << (clx::base64::encode(id) + "\r\n");
-			this->xresponse(ss);
-			ss << (clx::base64::encode(pass) + "\r\n");
-			this->xresponse(ss);
+			std::size_t len = id.size() * 2 + pass.size() + 2;
+			std::vector<char> buffer(len, 0);
+			std::memcpy(reinterpret_cast<char*>(&buffer.at(0)), id.c_str(), id.size());
+			std::memcpy(reinterpret_cast<char*>(&buffer.at(id.size() + 1)), id.c_str(), id.size());
+			std::memcpy(reinterpret_cast<char*>(&buffer.at((id.size() + 1) * 2)), pass.c_str(), pass.size());
+			
+			std::basic_string<char_type> cmd("AUTH PLAIN ");
+			clx::base64::encode(buffer.begin(), buffer.end(), std::inserter(cmd, cmd.end()));
+			this->command(ss, cmd);
 			
 			return *this;
 		}
 		
-		basic_smtp& xcram_md5(const string_type& id, const string_type& pass) {
-			basic_sockbuf<socket_type, char_type> sbuf(*this);
-			std::basic_iostream<char_type> ss(&sbuf);
+		basic_smtp& auth_login(const string_type& id, const string_type& pass) {
+			std::basic_iostream<char_type> ss(&sbuf_);
 			
-			ss << "AUTH CRAM-MD5\r\n";
-			this->xresponse(ss);
+			this->command(ss, string_type("AUTH LOGIN"));
+			this->command(ss, clx::base64::encode(id));
+			this->command(ss, clx::base64::encode(pass));
+			
+			return *this;
+		}
+		
+		basic_smtp& auth_cram_md5(const string_type& id, const string_type& pass) {
+			std::basic_iostream<char_type> ss(&sbuf_);
+			
+			this->command(ss, string_type("AUTH CRAM-MD5"));
 			
 			string_type buf = base64::decode(this->response(0).second);
 			buf = id + " " + hmac<md5>(pass, buf).to_string();
-			ss << (base64::encode(buf) + "\r\n");
-			this->xresponse(ss);
+			this->command(ss, base64::encode(buf));
 			
 			return *this;
 		}
 	};
 	
-	typedef basic_smtp<tcp::socket> smtp;
+	typedef clx::basic_smtp<clx::tcp::socket> smtp;
 }
 
 #endif // CLX_SMTP_H
