@@ -29,7 +29,7 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  Last-modified: Wed 02 Sep 2009 02:02:00 JST
+ *  Last-modified: Tue 03 Nov 2009 21:53:03 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_TOKENIZER_H
@@ -37,7 +37,10 @@
 
 #include <string>
 #include <vector>
+#include "config.h"
 #include "tokenizer_func.h"
+#include "container_generator.h"
+#include "container_accessor.h"
 
 namespace clx {
 	/* --------------------------------------------------------------------- */
@@ -48,95 +51,82 @@ namespace clx {
 		class Type = std::basic_string<char>,
 		class Container = std::vector<Type>
 	>
-	class basic_tokenizer {
+	class basic_tokenizer :
+		private container_generator<Container>,
+		public random_accessor<Container> {
+	private:
+		typedef container_generator<Container> generator;
+		typedef random_accessor<Container> accessor;
 	public:
 		typedef TokenizerFunc functor;
 		typedef typename functor::char_type char_type;
 		typedef std::basic_string<char_type> string_type;
-		
 		typedef Type value_type;
 		typedef Container container;
-		typedef typename container::size_type size_type;
-		typedef typename container::const_iterator const_iterator;
-		typedef typename container::iterator iterator;
 		
 		// constructor and destructor
+		basic_tokenizer(const basic_tokenizer& cp) :
+			generator(cp.member), accessor(this->member), f_(cp.f_) {}
+		
 		explicit basic_tokenizer(functor f = functor()) :
-			v_(), f_(f) {}
+			generator(), accessor(this->member), f_(f) {}
 		
 		template <class InIter>
-		explicit basic_tokenizer(InIter first, InIter last, functor f = functor()) :
-			v_(), f_(f) { this->assign(first, last); }
+		basic_tokenizer(InIter first, InIter last, functor f = functor()) :
+			generator(), accessor(this->member), f_(f) { this->assign(first, last); }
 		
 		explicit basic_tokenizer(const string_type& s, functor f = functor()) :
-			v_(), f_(f) { this->assign(s); }
+			generator(), accessor(this->member), f_(f) { this->assign(s); }
 		
 		explicit basic_tokenizer(const char_type* s, functor f = functor()) :
-			v_(), f_(f) { this->assign(s); }
+			generator(), accessor(this->member), f_(f) { this->assign(s); }
 		
-		virtual ~basic_tokenizer() {}
+		virtual ~basic_tokenizer() throw() {}
+		
+		basic_tokenizer& operator=(const basic_tokenizer& cp) {
+			this->member = cp.member;
+			f_ = cp.f_;
+			return *this;
+		}
 		
 		//operator
 		void reset() {
-			v_.clear();
+			this->member.clear();
 			f_.reset();
 		}
 		
 		template <class InIter>
-		const container& assign(InIter first, InIter last) {
+		basic_tokenizer& assign(InIter first, InIter last) {
 			value_type token;
 			
 			this->reset();
 			while (f_(first, last, token)) {
-				v_.insert(v_.end(), token);
+				this->member.insert(this->member.end(), token);
 				if (first == last) break;
 				token = value_type();
 			}
-			return v_;
+			return *this;
 		}
 		
-		const container& assign(const string_type& s) {
-			if (s.empty()) return v_;
+		basic_tokenizer& assign(const string_type& s) {
+			if (s.empty()) return *this;
 			return this->assign(s.begin(), s.end());
 		}
 		
-		const container& assign(const char_type* s) {
+		basic_tokenizer& assign(const char_type* s) {
 			string_type tmp(s);
 			return this->assign(tmp);
 		}
 		
 		template <class InIter>
-		const container& operator()(InIter first, InIter last) {
+		basic_tokenizer& operator()(InIter first, InIter last) {
 			return this->assign(first, last);
 		}
 		
-		const container& operator()(const string_type& s) { return this->assign(s); }
-		const container& operator()(const char_type* s) { return this->assign(s); }
-		
-		// inherit container methods
-		const value_type& operator[](size_type pos) const { return v_[pos]; }
-		const value_type& at(size_type pos) const { return v_.at(pos); }
-		const value_type& front() const { return v_.front(); }
-		const value_type& back() const { return v_.back(); }
-		
-		bool empty() const { return v_.empty(); }
-		size_type size() const { return v_.size(); }
-		
-		const_iterator begin() const { return v_.begin(); }
-		const_iterator end() const { return v_.end(); }
-		const_iterator rbegin() const { return v_.rbegin(); }
-		const_iterator rend() const { return v_.rend(); }
-		
-		iterator begin() { return v_.begin(); }
-		iterator end() { return v_.end(); }
-		iterator rbegin() { return v_.rbegin(); }
-		iterator rend() { return v_.rend(); }
-		
-		// get container
-		const container& data() const { return v_; }
+		basic_tokenizer& operator()(const string_type& s) { return this->assign(s); }
+		basic_tokenizer& operator()(const char_type* s) { return this->assign(s); }
 		
 	private:
-		container v_;
 		functor f_;
 	};
 	
