@@ -2,7 +2,7 @@
 /*
  *  timer.h
  *
- *  Copyright (c) 2004 - 2008, clown. All rights reserved.
+ *  Copyright (c) 2004 - 2009, clown. All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions
@@ -29,17 +29,20 @@
  *  NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *  SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  Last-modified: Thu 18 Dec 2008 11:08:00 JST
+ *  Last-modified: Wed 18 Nov 2009 21:58:03 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_WIN32_TIMER_H
 #define CLX_WIN32_TIMER_H
 
+#include "../config.h"
 #include <windows.h>
 
+#ifdef CLX_ENABLE_AUTOLINK
 #if defined(_MSC_VER) && (_MSC_VER >= 1200)
 #pragma comment(lib, "winmm.lib")
 #endif
+#endif // CLX_ENABLE_AUTOLINK
 
 namespace clx {
 	namespace detail {
