@@ -2,7 +2,7 @@
 /*
  *  zbuf.h
  *
- *  Copyright (c) 2004 - 2009, clown. All rights reserved.
+ *  Copyright (c) 2004 - 2010, clown. All rights reserved.
  *
  *  Redistribution and use in source and binary forms, with or without
  *  modification, are permitted provided that the following conditions
@@ -34,7 +34,7 @@
  *  The source code is derived from zlib.
  *  (See the following URL: http://zlib.net/zlib_license.html)
  *
- *  Last-modified: Thu 02 Apr 2009 10:10:00 JST
+ *  Last-modified: Wed 02 Jun 2010 04:28:00 JST
  */
 /* ------------------------------------------------------------------------- */
 #ifndef CLX_ZBUF_H
@@ -47,14 +47,16 @@
 #include <vector>
 #include <zlib.h>
 
+#ifdef CLX_ENABLE_AUTOLINK
 #if defined(_MSC_VER) && (_MSC_VER >= 1200)
 #pragma comment(lib, "zlib.lib")
 #endif
+#endif // CLX_ENABLE_AUTOLINK
 
 namespace clx {
 	/* --------------------------------------------------------------------- */
 	/*
-	 *  basic_zcompressbuf
+	 *  basic_zstreambuf
 	 *
 	 *  CompressRate can be set to the value from zero to nine.
 	 *  Note that Z_DEFAULT_COMPRESSION means six.
@@ -65,7 +67,7 @@ namespace clx {
 		class CharT = char,
 		class Traits = std::char_traits<CharT>
 	>
-	class basic_zcompressbuf : public std::basic_streambuf<CharT, Traits> {
+	class basic_zstreambuf : public std::basic_streambuf<CharT, Traits> {
 	public:
 		typedef unsigned char byte_type;
 		typedef CharT char_type;
@@ -75,7 +77,7 @@ namespace clx {
 		typedef typename container::size_type size_type;
 		typedef typename Traits::int_type int_type;
 		
-		basic_zcompressbuf(stream_type& out, size_type n) :
+		basic_zstreambuf(stream_type& out, size_type n) :
 			out_(out), buffer_(n, 0), z_(), flush_(Z_NO_FLUSH) {
 			z_.zalloc = Z_NULL;
 			z_.zfree = Z_NULL;
@@ -85,7 +87,7 @@ namespace clx {
 			z_.avail_out = buffer_.size() * sizeof(char_type);
 		}
 		
-		virtual ~basic_zcompressbuf() throw() {
+		virtual ~basic_zstreambuf() throw() {
 			this->finish();
 			deflateEnd(&z_);
 		}
@@ -108,7 +110,7 @@ namespace clx {
 		
 		virtual std::streamsize xsputn(const char_type* s, std::streamsize n) {
 			z_.next_in = reinterpret_cast<byte_type*>(const_cast<char_type*>(s));
-			z_.avail_in = n;
+			z_.avail_in = static_cast<uInt>(n);
 			
 			while (flush_ == Z_FINISH || z_.avail_in > 0) {
 				int status = deflate(&z_, flush_);
@@ -140,13 +142,13 @@ namespace clx {
 	};
 	
 	/* --------------------------------------------------------------------- */
-	//  basic_zdecompressbuf
+	//  basic_unzstreambuf
 	/* --------------------------------------------------------------------- */
 	template <
 		class CharT = char,
 		class Traits = std::char_traits<CharT>
 	>
-	class basic_zdecompressbuf : public std::basic_streambuf<CharT, Traits> {
+	class basic_unzstreambuf : public std::basic_streambuf<CharT, Traits> {
 	public:
 		typedef unsigned char byte_type;
 		typedef CharT char_type;
@@ -156,7 +158,7 @@ namespace clx {
 		typedef typename container::size_type size_type;
 		typedef typename Traits::int_type int_type;
 		
-		basic_zdecompressbuf(stream_type& in, size_type n) :
+		basic_unzstreambuf(stream_type& in, size_type n) :
 			in_(in), sbuf_(n, 0), dbuf_(n, 0), z_(), flush_(Z_NO_FLUSH) {
 			this->setg(&dbuf_[0], &dbuf_[0], &dbuf_[0]);
 			z_.zalloc = Z_NULL;
@@ -167,7 +169,7 @@ namespace clx {
 			inflateInit(&z_);
 		}
 		
-		virtual ~basic_zdecompressbuf() throw() { inflateEnd(&z_); }
+		virtual ~basic_unzstreambuf() throw() { inflateEnd(&z_); }
 		
 	protected:
 		virtual int_type underflow() {
@@ -179,6 +181,12 @@ namespace clx {
 			return traits::to_int_type(*this->gptr());
 		}
 		
+		virtual int_type uflow() {
+			int_type c = this->underflow();
+			if (!traits::eq_int_type(c, traits::eof())) this->gbump(1);
+			return c;
+		}
+		
 	private:
 		stream_type& in_;
 		container sbuf_;
